/*
	     .o        88   oooooooo      88  .ooooo.    ooooooooo
	   .d88       .8'  dP"""""""     .8' 888' `Y88. d"""""""8'
	 .d'888      .8'  d88888b.      .8'  888    888       .8'
       .d'  888     .8'       `Y88b    .8'    `Vbood888      .8'
       88ooo888oo  .8'          ]88   .8'          888'     .8'
	    888   .8'     o.   .88P  .8'         .88P'     .8'
	   o888o  88      `8bd88P'   88        .oP'       .8'

     []-------------------------------------------------------------[]
     |		                 CIRC.C                              |
     |						 	             |
     |           Teste de caminhada sobre uma curva plana            |
     |						                     |
     |                por Lenimar N. Andrade, maio/97                |
     []-------------------------------------------------------------[]

Para compilar:
 tcc -2 -DELIPSE -ecirc1.exe circ.c graphics.lib
 tcc -2 -DPARABOLA -ecirc2.exe circ.c graphics.lib
 tcc -2 -DTREVO3FOLHAS -ecirc3.exe circ.c graphics.lib
 tcc -2 -DCUBICO -ecirc4.exe circ.c graphics.lib
 tcc -2 -DCARDIOIDE -ecirc5.exe circ.c graphics.lib
 tcc -2 -DPAUSA -DVALORES -DELIPSE -e_circ1.exe circ.c graphics.lib
 tcc -2 -DPAUSA -DVALORES -DPARABOLA -e_circ2.exe circ.c graphics.lib
 tcc -2 -DPAUSA -DVALORES -DTREVO3FOLHAS -e_circ3.exe circ.c graphics.lib
 tcc -2 -DPAUSA -DVALORES -DCUBICO -e_circ4.exe circ.c graphics.lib
 tcc -2 -DPAUSA -DVALORES -DCARDIOIDE -e_circ5.exe circ.c graphics.lib

*/

#define GRAF2D
#include "graf.c"

#define COR_CURVA            DARKGRAY
#define COR_CIRC             CYAN
#define COR_PONTOS_NA_CURVA  RED
#define COR_PASSO_TANGENCIAL LIGHTMAGENTA
#define COR_PASSO_CIRCULAR   WHITE

/* #define ELIPSE */
/* #define PARABOLA */
/* #define TREVO3FOLHAS */
/* #define CUBICO */
/* #define CARDIOIDE */

#define DEMORA         /* Nao espera ser pressionada tecla para continuar */
/* #define PAUSA */
/* #define VALORES */

#ifdef PAUSA
#undef DEMORA
#endif

#ifdef CARDIOIDE
double f1(double t) { return cos(t)*(1 + 2*sin(t)); }
double f2(double t) { return sin(t)*(1 + 2 *sin(t)) - 1; }
double f3(double t) { return 0; }
#endif

#ifdef CUBICO
double f2(double t) { return t*t*t/6 - 3*t*t/4 + t; }
double f1(double t) { return t; }
double f3(double t) { return 0; }
#endif

#ifdef ELIPSE
double f1(double t) { return 3*cos(t); }
double f2(double t) { return cos(t) - 2*sin(t); }
double f3(double t) { return 0; }
#endif

#ifdef PARABOLA
double f1(double t) { return t; }
double f2(double t) { return t*t/3 - 2; }
double f3(double t) { return 0; }
#endif

#ifdef TREVO3FOLHAS
double f1(double t) { return sin(t) + cos(2*t); }
double f2(double t) { return cos(t) + sin(2*t); }
double f3(double t) { return 0; }
#endif

void main(int argc, char *argv[]) {

  ponto C, R, T, P, Q;
  vetor v1, v2, v;
  char ch, equacoes[100], mens[100], nome_arq[30] = "DADOS.TXT", 
       SALVAR_ARQ_TEXTO = 1;
  double k, norma_v, n, incr, t1, t2, tmin, tmax, r, distR,
	 distT, tT, tR,
	 t_incr = 0.1,    /* Intervalo entre t1 e t2 (na curva)     */
	 d_incr = 0.01,   /* Usado para calcular distancia `a curva */
	 L = 0.3;         /* Tamanho do passo da caminhada          */
  FILE *arq;
  #ifdef DEMORA
  int demora = 200;       /* Milisegundos entre duas amostragens    */
  if (argc > 1) demora = atoi(argv[1]);
  #endif
  
  if ((arq = fopen(nome_arq, "wt")) == NULL) {
    printf("ERRO: Arquivo %s nao pode ser aberto", nome_arq);
    return;
  }

  #ifdef CARDIOIDE
  strcpy(equacoes, "f(t) = (cos(t)(1+2sen(t)), sen(t)(1+2sen(t)) - 1)");
  TAM = 80;
  tmin = -PI, tmax = PI;
  n = 50;
  #endif

  #ifdef ELIPSE
  strcpy(equacoes, "f(t) = (3cos(t), cos(t) - 2sen(t))");
  TAM = 60;
  tmin = -PI, tmax = PI;
  n = 80;
  #endif

  #ifdef CUBICO
  strcpy(equacoes, "f(t) = (t, t*t*t/6 - 3*t*t/4 + t)");
  TAM = 30;
  tmin = -2, tmax = 5;
  n = 80;
  #endif

  #ifdef PARABOLA
  strcpy(equacoes, "f(t) = (t, t*t/3 - 2)");
  TAM = 50;
  tmin = -4, tmax = 4;
  n = 50;
  #endif

  #ifdef TREVO3FOLHAS
  strcpy(equacoes, "f(t) = (sen(t) + cos(2t), cos(t) + sen(2t))");
  TAM = 70;
  tmin = -PI, tmax = PI;
  n = 80;
  #endif

  incr = (tmax - tmin)/n;
  t1 = tmin;
  t2 = t1 + t_incr;
  fprintf(arq, "%s\n", equacoes);
  fprintf(arq, "L = %5.5f\n", L);

  if (!IniciaModoGrafico()) {
    printf("\nErro na mudanca para modo grafico\n");
    return;
  }

  do {

    P.x = f1(t1), P.y = f2(t1), P.z = f3(t1);
    v1.x = DER(f1,t1,h_erro), v1.y = DER(f2,t1,h_erro),
    v1.z = DER(f3,t1,h_erro);
    Q.x = f1(t2), Q.y = f2(t2), Q.z = f3(t2);
    v2.x = DER(f1,t2,h_erro), v2.y = DER(f2,t2,h_erro),
    v2.z = DER(f3,t2,h_erro);

    k = 1/Curvatura(f1, f2, f3, t2);

    if (!CentroDaCircunferencia(&C, &r, P, v1, Q, v2)) {
      printf("\nErro no calculo do centro da circunferencia\n");
      return;
    }

    if (r < 1) {
      if (!PontoNaCircunferencia(&R, P, v1, Q, v2, C, L)) {
	printf("\nErro no calculo do ponto sobre a circunferencia\n");
	return;
      }
    } else {
      if (!PontoNaCircunferencia(&R, P, v1, Q, v2, C, L/r)) {
	printf("\nErro no calculo do ponto sobre a circunferencia\n");
	return;
      }
    }

    VetorTangenteACurva(&v, f1, f2, f3, t2);
    norma_v = NORMA(v.x, v.y, v.z);
    v.x /= norma_v, v.y /= norma_v, v.z /= norma_v;
    T.x = Q.x + L*v.x, T.y = Q.y + L*v.y, T.z = Q.z + L*v.z;

    XCentro = C.x, YCentro = C.y, ZCentro = C.z, Raio = r;

    distR = Dist(R, f1, f2, f3, tmin, tmax, d_incr, &tR);
    distT = Dist(T, f1, f2, f3, tmin, tmax, d_incr, &tT);

    DesenhaGraficoParam(g1, g2, 0, 2*PI, COR_CURVA);
    DesenhaGraficoParam(f1, f2, tmin, tmax, COR_CIRC);
    MarcaPonto(C.x, C.y, COR_CURVA);
    MarcaPonto(P.x, P.y, COR_PONTOS_NA_CURVA);
    MarcaPonto(Q.x, Q.y, COR_PONTOS_NA_CURVA);
    MarcaPonto(R.x, R.y, COR_PASSO_CIRCULAR);
    MarcaPonto(T.x, T.y, COR_PASSO_TANGENCIAL);

    #ifdef VALORES
    setcolor(MAGENTA);
    sprintf(mens, "dist(Tangencial) = %5.5f", distT);
    outtextxy(0, 0, mens);
    setcolor(COR_PASSO_CIRCULAR);
    sprintf(mens, "dist(Circular) = %5.5f", distR);
    outtextxy(0, 10, mens);
    setcolor(COR_CURVA);
    sprintf(mens, "R = %5.5f  1/k = %5.5f", r, k);
    outtextxy(0, 20, mens);
    sprintf(mens, "t = %5.5f", t2);
    setcolor(COR_PONTOS_NA_CURVA);
    outtextxy(0, 30, mens);
    #endif

    if (SALVAR_ARQ_TEXTO) {
      fprintf(arq, "\ndist(Tangencial) = %5.5f", distT);
      fprintf(arq, "\ndist(Circular) = %5.5f", distR);
      fprintf(arq, "\nR = %5.5f  1/k = %5.5f", r, k);
      fprintf(arq, "\nt = %5.5f\n", t2);
    }
    
    #ifdef PAUSA
    ch = getch();
    if (ch == 0) ch = getch();
    else if (ch == 27) {
      closegraph();
      fclose(arq);
      return;
    }
    #endif

    #ifdef DEMORA
    if (kbhit()) {
      closegraph();
      fclose(arq);
      return;
    }
    delay(demora);
    #endif

    /* Apaga o grafico desenhado anteriormente */
    setcolor(0);
    DesenhaGraficoParam(g1, g2, 0, 2*PI, 0);
    MarcaPonto(C.x, C.y, 0);
    MarcaPonto(P.x, P.y, 0);
    MarcaPonto(Q.x, Q.y, 0);
    MarcaPonto(R.x, R.y, 0);
    MarcaPonto(T.x, T.y, 0);

    #ifdef VALORES
    sprintf(mens, "dist(Tangencial) = %5.5f", distT);
    outtextxy(0, 0, mens);
    sprintf(mens, "dist(Circular) = %5.5f", distR);
    outtextxy(0, 10, mens);
    sprintf(mens, "R = %5.5f  1/k = %5.5f", r, k);
    outtextxy(0, 20, mens);
    sprintf(mens, "t = %5.5f", t2);
    outtextxy(0, 30, mens);
    #endif

    t1 += incr;
    t2 += incr;
    if (t2 > tmax) SALVAR_ARQ_TEXTO = 0; 

  } while (!kbhit());

  closegraph();
  fclose(arq);
}

/*** FIM DE CIRC.C ***/
